library(ggplot2)
library(ncdf4)
library(trend)
library(loadeR)
library(transformeR)
library(visualizeR)
library(RColorBrewer)

rm(list=ls()) ; gc()

#Working directory
dir <- "/esarchive/scratch/molmo/CLIMCAT_MJ/CORDEX/"
OUTPUTS <- "/esarchive/scratch/molmo/CLIMCAT_MJ/OUTPUTS/"
setwd(dir)

invariable <- loadGridData(dataset = "/esarchive/scratch/molmo/mask_0.1_EUROCORDEX.nc",
                           var="sftlf") 

#interpGrid(invariable,new.coordinates = list(x=seq(-9.5,24.5,by=0.1),y=seq(33.5,44.5,by=0.1))) %>%
invariable <-  binaryGrid(invariable,condition = "GT",threshold=0)

scenario <- c("rcp26","rcp45","rcp85")

for(sc in 1:length(scenario)){
  
  #-------Annual indices---------#
  #Useful information
  indices <- c("wsdi","csdi","tr",
               "r20mm","prcptot","cdd","cwd")
  files <- intersect(list.files(pattern="2100.nc",recursive = T),list.files(pattern="yr",recursive = T))
  files <- intersect(files,list.files(pattern=scenario[sc],recursive=T))
  files2 <- intersect(list.files(pattern="2099.nc",recursive=T),list.files(pattern="yr",recursive=T))
  files2 <- intersect(files2,list.files(pattern=scenario[sc],recursive=T))
  # files3 <- intersect(list.files(pattern="ERA5_NULL_NULL_1950-2022.nc",recursive=T),list.files(pattern="yr",recursive=T))
  
  files <- c(files,files2)#,files3)
  files <- intersect(files,list.files(pattern="r1i1p1",recursive=T))  
 
   ssn <- "Annual"
  seasons <- list(c(1:12))
  Model <- NA
  
  series.indices <- list()
  
  for(i in 1:length(indices)){
    print(indices[i])
    files_subset <- intersect(files,list.files(pattern=indices[i],recursive=T))
    series.indices[[i]] <- list()
    
    for(m in 1:length(files_subset)){
      print(files_subset[m])
      
      series.indices[[i]][[m]] <- list()
      
      for(s in 1:length(seasons)){
        di <- dataInventory(files_subset[m])
        
        ini <- nchar(names(di))                            
        out <- nchar("_historical-ssp585_r1i1p1f1_1950-2100.nc") 
        Model[m] <- substr(files_subset[m],start=ini+32,stop=nchar(files_subset[m])-out)
        
        data <- loadGridData(files_subset[m],var=names(di),years=1950:2100)
        
        #Interpolate into a common grid and mask ocean
        data <- interpGrid(data,new.coordinates = getGrid(invariable),method = "nearest")
        for(y in 1:dim(data$Data)[1]){
          data$Data[y,,] <- data$Data[y,,]*invariable$Data[1,,]
        }
        
        #Estimating anomalies wrt 1981-2010 reference period
        data <- scaleGrid(data,base = subsetGrid(data,years=1981:2010),type = "center")
        data <- climatology(subsetYears(data,seq(2071,getYearsAsINDEX(data)[length(getYearsAsINDEX(data))])))
        
        #Serie regional
        # EuroMediterranean <- aggregateGrid(data,aggr.spatial = list(FUN="mean",na.rm=T))
        # Catalonia <- aggregateGrid(subsetGrid(data,latLim = c(40,43),lonLim = c(0,4)),aggr.spatial = list(FUN="mean",na.rm=T))
        
        Map <- data.frame(Value=c(data$Data),
                          Season=ssn[s],
                          Index=indices[i],
                          Model=Model[m],
                          Longitude=rep(data$xyCoords$x,each=length(data$xyCoords$y)),
                          Latitude=rep(data$xyCoords$y,length(data$xyCoords$x)))
        
        if(indices[i]%in%c("cdd")){paleta=rev(brewer.pal(11,"BrBG")); range=c(-max(abs(range(Map$Value,na.rm=T))),max(abs(range(Map$Value,na.rm=T))))}
        if(indices[i]%in%c("cwd","r20mm")){paleta=c(brewer.pal(11,"BrBG")); range=c(-max(abs(range(Map$Value,na.rm=T))),max(abs(range(Map$Value,na.rm=T))))}
        if(indices[i]%in%c("prcptot")){paleta=brewer.pal(11,"BrBG"); range=c(-max(abs(range(Map$Value,na.rm=T))),max(abs(range(Map$Value,na.rm=T))))}
        if(indices[i]%in%c("wsdi","tr")){paleta=rev(brewer.pal(11,"RdYlBu")); range=c(-max(abs(range(Map$Value,na.rm=T))),max(abs(range(Map$Value,na.rm=T))))}
        if(indices[i]%in%c("csdi")){paleta=brewer.pal(11,"RdYlBu"); range=c(-max(abs(range(Map$Value,na.rm=T))),max(abs(range(Map$Value,na.rm=T))))}
        
        # png(filename = paste(OUTPUTS,"DeltaChange_CORDEX_",ssn[s],"_",indices[i],"_",Model[m],"_Historical+",scenario[sc],".png",sep=""),res=600,width=3000,height=2000)
        # print(ggplot(data=Map,aes(x=Longitude,y=Latitude,fill=Value)) + geom_tile()  +
        #         scale_fill_gradientn(name=paste(di[[names(di)]][["Units"]]),colours=paleta,limits=range,na.value = "white")+
        #         coord_quickmap(xlim=c(min(Map$Longitude),max(Map$Longitude)),
        #                        ylim=c(min(Map$Latitude),max(Map$Latitude)),expand=F)+ borders() +
        #         theme_bw() + ggtitle(paste(ssn[s],indices[i],Model[m],"2071-2100",sep=" ")))#+geom_text(data=subset(datita,Significance<=0.05),aes(label="*",fontface="bold")))
        # dev.off()
        
        series.indices[[i]][[m]][[s]] <- Map
        gc()
        
      }
    }
  }
  save(series.indices,file=paste("map.indices.yr.CORDEX.",scenario[sc],".Rdata",sep=""))
  
  #--------Seasonal indices--------------------#
  files <- intersect(list.files(pattern="210012.nc",recursive = T),list.files(pattern="mon",recursive = T))
   files <- intersect(files,list.files(pattern=scenario[sc],recursive=T))
   files2 <- intersect(list.files(pattern="209912.nc",recursive=T),list.files(pattern="mon",recursive=T))
   files2 <- intersect(files2,list.files(pattern=scenario[sc],recursive=T))
   #files3 <- intersect(list.files(pattern="ERA5_NULL_NULL_195001-202212.nc",recursive=T),list.files(pattern="mon",recursive=T))

   files <- c(files,files2)#,files3)
   files <- intersect(files,list.files(pattern="r1i1p1",recursive=T))
  
  seasons <- list(c(12,1,2),c(3,4,5),c(6,7,8),c(9,10,11)) #DJF #MAM #JJA #SON #ANNUAL
  ssn <- c("Winter","Spring","Summer","Autumn")#,"Annual")
  indices <- c("txx","txn","tnx","tnn","tx90p","tx10p","tn90p","tn10p",
               "rx5day","rx1day")
  Model <- NA
  
  series.indices <- list()
  
  for(i in 1:length(indices)){
    print(indices[i])
    files_subset <- intersect(files,list.files(pattern=indices[i],recursive = T))
    
    series.indices[[i]] <- list()
    
    for(m in 1:length(files_subset)){
      print(files_subset[m])
      
      series.indices[[i]][[m]] <- list()
      
      for(s in 1:length(seasons)){
        di <- dataInventory(files_subset[m])
        
        ini <- nchar(names(di))                            
        out <- nchar("_historical-ssp585_r1i1p1f1_195001-210012.nc") 
        Model[m] <- substr(files_subset[m],start=ini+33,stop=nchar(files_subset[m])-out)
        
        data <- loadGridData(files_subset[m],var=names(di),years=1950:2100)
        
        #Interpolate into a common grid and mask ocean
        data <- interpGrid(data,new.coordinates = getGrid(invariable),method = "nearest")
        for(y in 1:dim(data$Data)[1]){
          data$Data[y,,] <- data$Data[y,,]*invariable$Data[1,,]
        }
        
        #Estimating anomalies wrt 1981-2010 reference period
        data <- scaleGrid(data,base = subsetGrid(data,years=1981:2010),type = "center",time.frame = "monthly")
        
        data <- aggregateGrid(data,aggr.s = list(FUN=list("mean",na.rm=T),season=seasons[[s]]))
        
        data <- climatology(subsetYears(data,seq(2071,getYearsAsINDEX(data)[length(getYearsAsINDEX(data))])))
        
        #Serie regional
        # EuroMediterranean <- aggregateGrid(data,aggr.spatial = list(FUN="mean",na.rm=T))
        # Catalonia <- aggregateGrid(subsetGrid(data,latLim = c(40,43),lonLim = c(0,4)),aggr.spatial = list(FUN="mean",na.rm=T))
        
        Map <- data.frame(Value=c(data$Data),
                          Season=ssn[s],
                          Index=indices[i],
                          Model=Model[m],
                          Longitude=rep(data$xyCoords$x,each=length(data$xyCoords$y)),
                          Latitude=rep(data$xyCoords$y,length(data$xyCoords$x)))
        
        if(di[[names(di)]][["Units"]]%in%c("mm")){paleta=brewer.pal(11,"BrBG"); range=c(-max(abs(range(Map$Value,na.rm=T))),max(abs(range(Map$Value,na.rm=T))))}
        if(di[[names(di)]][["Units"]]%in%c("degrees_C","days","%")){paleta=rev(brewer.pal(11,"RdBu")); range=c(-max(abs(range(Map$Value,na.rm=T))),max(abs(range(Map$Value,na.rm=T))))}
        
        # png(filename = paste(OUTPUTS,"DeltaChange_CMIP6_",ssn[s],"_",indices[i],"_",Model[m],"_Historical+",scenario[sc],".png",sep=""),res=600,width=3000,height=2000)
        # print(ggplot(data=Map,aes(x=Longitude,y=Latitude,fill=Value)) + geom_tile()  +
        #         scale_fill_gradientn(name=paste(di[[names(di)]][["Units"]]),colours=paleta,limits=range,na.value = "white")+
        #         coord_quickmap(xlim=c(min(Map$Longitude),max(Map$Longitude)),
        #                        ylim=c(min(Map$Latitude),max(Map$Latitude)),expand=F)+ borders() +
        #         theme_bw() + ggtitle(paste(ssn[s],indices[i],Model[m],"2071-2100",sep=" ")))#+geom_text(data=subset(datita,Significance<=0.05),aes(label="*",fontface="bold")))
        # dev.off()
        
        series.indices[[i]][[m]][[s]] <- Map
        gc()
        
      }
    }
  }
  save(series.indices,file=paste("map.indices.mon.CORDEX.",scenario[sc],".Rdata",sep=""))
}
