rm(list=ls()) ; graphics.off() ; gc()

#Load libraries
library(ncdf4)
library(ggplot2)
library(gridExtra)
library(fields)
library(mapdata)
library(matrixStats)
library(metR)
library(RColorBrewer)
library(fields)
library(metR)
library(transformeR)
library(loadeR)
#library(climate4R.value)
library(visualizeR)
#library(trend)
library(plotrix)
#library(miceadds)

rm(list=ls()) ; gc()

#Directory information
setwd("../../../scratch/")
OUTPUTS <- "/esarchive/scratch/molmo/CPs_SLP_ERA5/"

#Useful functions
png.save <- function(x,filename,main,xlab,ylab){
  png(filename = paste(filename,".png",sep=""))
  plot(x,type="o",main=main,xlab = xlab,ylab=ylab)
  dev.off()
}

avgmean <- function(x,ventana){
  value <- rep(NA,length(x))
  for(i in (1+ventana):(length(x)-ventana)){
    value[i] <- mean(x[(i-ventana):(i+ventana)],na.rm=T)
  }
  return(value)
}
####################################
#Load ERA5 data
message("Loading and preprocessing data...")
nc <- nc_open("psl_5022_1x1_D3.nc")
nc_lon <- ncvar_get(nc,"lon")
nc_lat <- ncvar_get(nc,"lat")
nc_var <- ncvar_get(nc,"psl") #x,y,time

#Selection of spatial domains
gridsize_x <- abs(nc_lon[2]-nc_lon[1])
gridsize_y <- abs(nc_lat[2]-nc_lat[1])

# domain1 <- list(lon <- which(nc_lon %in% seq(-15,15,by=gridsize_x)),
#                 lat <- which(nc_lat %in% seq(32,48,by=gridsize_y))) #red domain D1 #Spain
# 
# domain2 <- list(lon <- which(nc_lon %in% seq(-10,25,by=gridsize_x)),
#                 lat <- which(nc_lat %in% seq(33,45,by=gridsize_y))) #red domain D2 PEP Mediterranean
# 
# domain3 <- list(lon <- which(nc_lon %in% seq(-35,35,by=gridsize_x)),
#                 lat <- which(nc_lat %in% seq(20,70,by=gridsize_y))) #red domain D3 #Europe
# 
# domain <- domain2 

#Organize and Standardize data
var <- matrix(nc_var,dim(nc_var)[1]*dim(nc_var)[2],dim(nc_var)[3]) #space,time
var <- scale(t(var),center=T,scale=T)
rm(nc_var) ; gc()

#Load clustering information
do <- "D2"
dom <- do[1]
k=5 

#Preparing data
longitude=rep(nc_lon,length(nc_lat))
latitude=rep(nc_lat,each=length(nc_lon))

Z <- list() 

load(file = paste("WardCluster_Dendrogram_","D2","_50km.Rdata",sep=""))
vector_cluster <- cutree(ward_clustering,k)

for(cp in 1:k){
  Z[[cp]] <-colMeans(var[which(vector_cluster==cp),],na.rm=T)
}

datitosERA5 <- data.frame(Longitude=rep(longitude,k),
                          Latitude=rep(latitude,k),
                          Z = unlist(Z),
                          CP = rep(1:k,each=length(unlist(Z))/k))
datitosERA5$CP <- factor(datitosERA5$CP,levels=c(1:k))

#Temporal cycle
calendario <- seq(as.Date("1950-01-01"),as.Date("2021-12-31"),by="day")
fechas <- calendario[format(calendario,"%m-%d") != "02-29"]
n_years <- length(fechas)/365

datos = vector_cluster
if(length(datos)!=length(fechas)){
  datos <- datos[which(calendario%in%fechas)]
  rm(calendario,fechas)
}

datos2 = data.frame(group=as.numeric( datos),
                    day=rep(1:365,n_years))

valueERA5 <- matrix(NA,365,k)
for(i in 1:365){
  for(j in 1:k){
    valueERA5[i,j] <- length(which(datos2$group==j & datos2$day==i))/n_years*100
  }
}

# avgERA5 <- matrix(NA,365,k)
# for(j in 1:k){
#   avgERA5[,j] <- avgmean(x=valueERA5[,j],ventana = 15)
# }

################################################################################
#Start working with models
files <- list.files(pattern="2014.nc",path = "/esarchive/scratch/molmo/output/recipe_20231114_085303/preproc/diagnostic_2/psl_d1/")
files2 <- files

files2[9] <- files[10]
files2[10] <- files[11]
files2[11] <- files[12]
files2[12] <- files[13]
files2[13] <- files[9]

files2[21] <- files[22]
files2[22] <- files[21]

files <- files2
filesW <- list.files(pattern="WardClustering",path = "/esarchive/scratch/molmo/output/")

errors <- list()
gcms <- list()

for(f in 1:length(files)){
  load(file = paste("/esarchive/scratch/molmo/output/",filesW[f],sep=""))
  
  nc <- nc_open(paste("/esarchive/scratch/molmo/output/recipe_20231114_085303/preproc/diagnostic_2/psl_d1/",files[f],sep=""))
  nc_lon <- ncvar_get(nc,"lon")
  nc_lat <- ncvar_get(nc,"lat")
  nc_var <- ncvar_get(nc,"psl")
  var <- matrix(nc_var,dim(nc_var)[1]*dim(nc_var)[2],dim(nc_var)[3])

  var <- scale(t(var),scale=T,center=T)
  
  message("Reordering ",gcm)
  
  vector_cluster <- cutree(ward_clustering,k)
  #vector_cluster <- vector_cluster[which(calendario%in%fechas)]
  
  Zm <- list()
  for(cp in 1:k){
    Zm[[cp]] <- colMeans(var[which(vector_cluster==cp),],na.rm=T)
  }
  
  lon <- nc_lon
  for(i in 1:length(lon)){
    if(lon[i]>180){
      lon[i] <- lon[i]-360
    }
  }
  lat <- nc_lat    
  
  xmn <- min(lon) ; xmx <- max(lon) ; ymn <- min(lat) ; ymx <- max(lat)
  longitude=rep(lon,length(nc_lat))
  latitude=rep(seq(ymn,ymx,gridsize_x),each=length(nc_lon))
  
  datitos <- data.frame(Longitude=rep(longitude,k),
                        Latitude=rep(latitude,k),
                        Z = unlist(Zm),
                        CP = rep(1:k,each=length(unlist(Zm))/k))
  datitos$CP <- factor(datitos$CP,levels=c(1:k))
  
  #Ordeno las longitudes del GCM como en ERA5 para luego correlacionarlos / dist euclidea
  #Ahora viene el ordenamiento
  lonERA <- datitosERA5$Longitude[1:3500]
  latERA <- datitosERA5$Latitude[1:3500]
  
  patronGCM <- list()
  for(j in 1:k){
    patronGCM[[j]] <- NA
    for(i in 1:length(longitude)){
      patronGCM[[j]][i] <- datitos$Z[which(datitos$Longitude==lonERA[i] & datitos$Latitude==latERA[i] & datitos$CP==j)]
    }
  }
  
  #Matriz de distancias
  distance <- matrix(NA,k,k)
  for(i in 1:k){
    for(j in 1:k){
      distance[i,j] <- dist(rbind(Z[[i]],patronGCM[[j]]),method = "euclidean")
    }
  }
  
  orden <- rep(0,k)
  for(i in 1:5){
    closest_index <- which.min(distance[,i])
    
    if (closest_index %in% orden){
      left <- c(1:5)[-which(c(1:5)%in%orden)]
      closest_index <- left[which.min(distance[left,i])]
    }
    orden[i] <- closest_index  
  }
  
  #Reordenamiento
  datos <- vector_cluster
  for(i in 1:k){
    datos[which(vector_cluster==i)] <- orden[i]
  }
  
  #Temporal cycle
  calendario <- seq(as.Date("1950-01-01"),as.Date("2014-12-31"),by="day")
  fechas <- calendario[format(calendario,"%m-%d") != "02-29"]
  n_years <- length(fechas)/365
  
  if(length(datos)!=length(fechas)){
    datos <- datos[which(calendario%in%fechas)]
    rm(calendario,fechas)
  }
  
  datos2 = data.frame(group=as.numeric( datos),
                      day=rep(1:365,n_years))
  
  if(gcm %in% c("ACCESS-CM2","CMCC-CM2-HR4","EC-Earth3-AerChem","KIOST-ESM","IPSL-CM6A-LR","MPI-ESM1-2-HR","MPI-ESM1-2-LR","NorESM2-LM","NorESM2-MM","TaiESM1")){ 
    datos3 <- datos2
    datos3$group[which(datos2$group==1)] <- 3
    datos3$group[which(datos2$group==3)] <- 1
    datos2 <- datos3
  }

  # if(gcm %in% c()){
  #   datos3 <- datos2
  #   datos3$group[which(datos2$group==3)] <- 5
  #   datos3$group[which(datos2$group==5)] <- 3
  #   datos2 <- datos3
  # }
  
  value <- matrix(NA,365,k)
  for(i in 1:365){
    for(j in 1:k){
      value[i,j] <- length(which(datos2$group==j & datos2$day==i))/n_years*100
    }
  }

  # avg <- matrix(NA,365,k)
  # for(j in 1:k){
  #   avg[,j] <- avgmean(x=value[,j],ventana = 15)
  # }

  ErrorGCM <- matrix(NA,365,k)
  for(i in 1:365){
    for(j in 1:k){
      ErrorGCM[i,j] <- abs(valueERA5[i,j] - value[i,j])#/valueERA5[k,j]*100
    }
  }
  
  errors[[f]] <- round(colMeans(ErrorGCM,na.rm = T),digits=1)
  gcms[[f]] <- gcm
}

################################################################################
#Heatmaps ErrorCycle
data = data.frame(Error=unlist(errors),GCM=rep(unlist(gcms),each=k),CP=rep(c(1:k),length(unlist(gcms))))

data.mean <- data.frame(Error=as.numeric(lapply(X=errors,FUN=mean)),GCM=unlist(gcms),CP=rep("Mean",length(unlist(gcms))))

#data$Error[which(data$Error > 20)] <- 20
data$GCM <- factor(data$GCM,levels=rev(unique(data$GCM)))
data <- rbind(data,data.mean)

p <- ggplot(data=data,aes(x=CP,y=GCM,fill=Error)) + geom_tile() + coord_cartesian(expand=F) + theme_bw()+
  scale_fill_gradientn(name="days",colours=brewer.pal(9,"YlOrRd"),limits=c(0,20)) + geom_text(aes(label=Error))
ggsave(p,filename=paste(OUTPUTS,"ErrorCycle_GCMs_",dom,"_","CPs.png",sep=""),device="png",
       dpi=600,width=5,height = 7) 
