library(ncdf4)
library(ggplot2)
library(gridExtra)
library(fields)
library(mapdata)
library(matrixStats)
library(metR)
library(RColorBrewer)
library(plotrix)

#Asignación espacial comparando el dominio más grande sobre Europa (D3)
rm(list=ls()) ; gc()

#Directory information
setwd("../../../scratch/")
OUTPUTS <- "/esarchive/scratch/molmo/CPs_SLP_ERA5/"

#Useful functions
td <- function (ref, model, add = FALSE, col = "red", pch = 19, pos.cor = TRUE,text,coltext,
                xlab = "Standard deviation", ylab = "", main = "Taylor Diagram", 
                show.gamma = TRUE, ngamma = 3, gamma.col = 8, sd.arcs = 0, 
                ref.sd = FALSE, sd.method = "sample", grad.corr.lines = c(0.2, 
                                                                          0.4, 0.6, 0.8, 0.9), pcex = 1, cex.axis = 0.75, normalize = FALSE, 
                mar = c(4, 3, 4, 3), ...) 
{
  grad.corr.full <- c(0, 0.2, 0.4, 0.6, 0.8, 0.9, 0.95, 0.99, 
                      1)
  R <- cor(ref, model,use = "pairwise.complete.obs",method = "pearson")
  if (is.list(ref)) 
    ref <- unlist(ref)
  if (is.list(model)) 
    ref <- unlist(model)
  SD <- function(x, subn) {
    meanx <- mean(x, na.rm = TRUE)
    devx <- x - meanx
    ssd <- sqrt(sum(devx * devx, na.rm = TRUE)/(length(x[!is.na(x)]) - 
                                                  subn))
    return(ssd)
  }
  subn <- sd.method != "sample"
  sd.r <- SD(ref, subn)
  sd.f <- SD(model, subn)
  if (normalize) {
    sd.f <- sd.f/sd.r
    sd.r <- 1
  }
  maxsd <- 1.5 * 1.35
  oldpar <- par("mar", "xpd", "xaxs", "yaxs")
  if (!add) {
    par(mar = mar)
    if (pos.cor) {
      if (nchar(ylab) == 0) 
        ylab = "Standard deviation"
      plot(0, xlim = c(0, maxsd * 1.1), ylim = c(0, maxsd * 
                                                   1.1), xaxs = "i", yaxs = "i", axes = FALSE, main = main,
           xlab = "", ylab = ylab, type = "n", cex = cex.axis, font=2,
           ...)
      mtext(xlab, side = 1, line = 2.3,cex = 0.75,font=2)
      if (grad.corr.lines[1]) {
        for (gcl in grad.corr.lines) lines(c(0, maxsd * 
                                               gcl), c(0, maxsd * sqrt(1 - gcl^2)), lty = 3)
      }
      segments(c(0, 0), c(0, 0), c(0, maxsd), c(maxsd, 
                                                0))
      axis.ticks <- pretty(c(0, maxsd),n=8) #Aca modifico las lineas de Standard desv
      axis.ticks <- axis.ticks[axis.ticks <= maxsd]
      axis(1, at = axis.ticks, cex.axis = cex.axis,font=2)
      axis(2, at = axis.ticks, cex.axis = cex.axis,font=2)
      if (sd.arcs[1]) {
        if (length(sd.arcs) == 1) 
          sd.arcs <- axis.ticks
        for (sdarc in sd.arcs) {
          xcurve <- cos(seq(0, pi/2, by = 0.01)) * sdarc
          ycurve <- sin(seq(0, pi/2, by = 0.01)) * sdarc
          lines(xcurve, ycurve, col = "lightblue", lty = 3)
        }
      }
      if (show.gamma[1]) {
        if (length(show.gamma) > 1) 
          gamma <- show.gamma
        else gamma <- pretty(c(0, maxsd), n = ngamma)[-1]
        if (gamma[length(gamma)] > maxsd) 
          gamma <- gamma[-length(gamma)]
        labelpos <- seq(45, 70, length.out = length(gamma))
        for (gindex in 1:length(gamma)) {
          xcurve <- cos(seq(0, pi, by = 0.03)) * gamma[gindex] + 
            sd.r
          endcurve <- which(xcurve < 0)
          endcurve <- ifelse(length(endcurve), min(endcurve) - 
                               1, 105)
          ycurve <- sin(seq(0, pi, by = 0.03)) * gamma[gindex]
          maxcurve <- xcurve * xcurve + ycurve * ycurve
          startcurve <- which(maxcurve > maxsd * maxsd)
          startcurve <- ifelse(length(startcurve), max(startcurve) + 
                                 1, 0)
          lines(xcurve[startcurve:endcurve], ycurve[startcurve:endcurve], 
                col = gamma.col)
          if (xcurve[labelpos[gindex]] > 0) 
            boxed.labels(xcurve[labelpos[gindex]], ycurve[labelpos[gindex]], 
                         gamma[gindex], border = FALSE,cex=0.75,font=2)
        }
      }
      xcurve <- cos(seq(0, pi/2, by = 0.01)) * maxsd
      ycurve <- sin(seq(0, pi/2, by = 0.01)) * maxsd
      lines(xcurve, ycurve)
      bigtickangles <- acos(seq(0.1, 0.9, by = 0.1))
      medtickangles <- acos(seq(0.05, 0.95, by = 0.1))
      smltickangles <- acos(seq(0.91, 0.99, by = 0.01))
      segments(cos(bigtickangles) * maxsd, sin(bigtickangles) * 
                 maxsd, cos(bigtickangles) * 0.97 * maxsd, sin(bigtickangles) * 
                 0.97 * maxsd)
      par(xpd = TRUE)
      if (ref.sd) {
        xcurve <- cos(seq(0, pi/2, by = 0.03)) * sd.r
        ycurve <- sin(seq(0, pi/2, by = 0.03)) * sd.r
        lines(xcurve, ycurve)
      }
      points(sd.r, 0, cex = pcex,pch=19,col="black")
      text(cos(c(bigtickangles, acos(c(0.95, 0.99)))) * 
             1.05 * maxsd, sin(c(bigtickangles, acos(c(0.95, 
                                                       0.99)))) * 1.05 * maxsd, c(seq(0.1, 0.9, by = 0.1), 
                                                                                  0.95, 0.99), cex = cex.axis,font=2)
      text(maxsd * 0.8, maxsd * 0.8, "Correlation", srt = 315, 
           cex = cex.axis,font=2)
      segments(cos(medtickangles) * maxsd, sin(medtickangles) * 
                 maxsd, cos(medtickangles) * 0.98 * maxsd, sin(medtickangles) * 
                 0.98 * maxsd)
      segments(cos(smltickangles) * maxsd, sin(smltickangles) * 
                 maxsd, cos(smltickangles) * 0.99 * maxsd, sin(smltickangles) * 
                 0.99 * maxsd)
    }
    else {
      x <- ref
      y <- model
      R <- cor(x, y, use = "pairwise.complete.obs",method = "pearson")
      E <- mean(x, na.rm = TRUE) - mean(y, na.rm = TRUE)
      xprime <- x - mean(x, na.rm = TRUE)
      yprime <- y - mean(y, na.rm = TRUE)
      sumofsquares <- (xprime - yprime)^2
      Eprime <- sqrt(sum(sumofsquares)/length(complete.cases(x)))
      E2 <- E^2 + Eprime^2
      if (add == FALSE) {
        maxray <- 1.5 * max(sd.f, sd.r)
        plot(c(-maxray, maxray), c(0, maxray), type = "n", 
             asp = 1, bty = "n", xaxt = "n", yaxt = "n", 
             xlim = c(-1.1 * maxray, 1.1 * maxray), xlab = xlab, 
             ylab = ylab, main = main, cex = cex.axis,font=2)
        discrete <- seq(180, 0, by = -1)
        listepoints <- NULL
        for (i in discrete) {
          listepoints <- cbind(listepoints, maxray * 
                                 cos(i * pi/180), maxray * sin(i * pi/180))
        }
        listepoints <- matrix(listepoints, 2, length(listepoints)/2)
        listepoints <- t(listepoints)
        lines(listepoints[, 1], listepoints[, 2])
        lines(c(-maxray, maxray), c(0, 0))
        lines(c(0, 0), c(0, maxray))
        for (i in grad.corr.lines) {
          lines(c(0, maxray * i), c(0, maxray * sqrt(1 - 
                                                       i^2)), lty = 3)
          lines(c(0, -maxray * i), c(0, maxray * sqrt(1 - 
                                                        i^2)), lty = 3)
        }
        for (i in grad.corr.full) {
          text(1.05 * maxray * i, 1.05 * maxray * sqrt(1 - 
                                                         i^2), i, cex = cex.axis, adj = cos(i)/2,font=2)
          text(-1.05 * maxray * i, 1.05 * maxray * sqrt(1 - 
                                                          i^2), -i, cex = cex.axis, adj = 1 - cos(i)/2,font=2)
        }
        seq.sd <- seq.int(0, 2 * maxray, by = (maxray/10))[-1]
        for (i in seq.sd) {
          xcircle <- sd.r + (cos(discrete * pi/180) * 
                               i)
          ycircle <- sin(discrete * pi/180) * i
          for (j in 1:length(xcircle)) {
            if ((xcircle[j]^2 + ycircle[j]^2) < (maxray^2)) {
              points(xcircle[j], ycircle[j], col = "darkgreen", 
                     pch = ".")
              if (j == 10) 
                text(xcircle[j], ycircle[j], signif(i, 
                                                    2), cex = cex.axis, col = "darkgreen", 
                     srt = 90,font=2)
            }
          }
        }
        seq.sd <- seq.int(0, maxray, length.out = 5)
        for (i in seq.sd) {
          xcircle <- cos(discrete * pi/180) * i
          ycircle <- sin(discrete * pi/180) * i
          if (i) 
            lines(xcircle, ycircle, lty = 3, col = "pink")
          text(min(xcircle), -0.06 * maxray, signif(i, 
                                                    2), cex = cex.axis, col = "lightblue",font=2)
          text(max(xcircle), -0.06 * maxray, signif(i, 
                                                    2), cex = cex.axis, col = "lightblue",font=2)
        }
        # text(0, -0.14 * maxray, "Standard Deviation", 
        #     cex = cex.axis, col = "blue")
        text(0, -0.22 * maxray, "Centered RMS Difference", 
             cex = cex.axis, col = "darkgreen",font=2)
        points(sd.r, 0, pch = 19, bg = "darkgreen", cex = pcex)
        text(0, 1.2 * maxray, "Correlation Coefficient", 
             cex = cex.axis,font=2)
      }
      S <- (2 * (1 + R))/(sd.f + (1/sd.f))^2
    }
  }
  points(sd.f * R, sd.f * sin(acos(R)), pch = pch, col = col, 
         cex = pcex)
  text(sd.f * R, sd.f * sin(acos(R)),  #the line to add
       labels=text, cex = 0.6, pos = 3,col=coltext,font = 2) #You can change the pos argument to your liking
  invisible(oldpar)
}

#########################################################
#Load ERA5 data
message("Loading and preprocessing data...")
nc <- nc_open("psl_5022_1x1_D3.nc")
nc_lon <- ncvar_get(nc,"lon")
nc_lat <- ncvar_get(nc,"lat")
nc_var <- ncvar_get(nc,"psl") #x,y,time

#Selection of spatial domains
gridsize_x <- abs(nc_lon[2]-nc_lon[1])
gridsize_y <- abs(nc_lat[2]-nc_lat[1])

#Organize and Standardize data
var <- t(matrix(nc_var,dim(nc_var)[1]*dim(nc_var)[2],dim(nc_var)[3])) #space,time
#var <- scale(t(var),center=T,scale=T)
rm(nc_var) ; gc()

#Load clustering information
do <- "D2"
dom <- do[1]
k=5 

#Preparing data
longitude=rep(nc_lon,length(nc_lat))
latitude=rep(nc_lat,each=length(nc_lon))

Z <- list() 

load(file = paste("WardCluster_Dendrogram_","D2","_50km.Rdata",sep=""))
vector_cluster <- cutree(ward_clustering,k)

for(cp in 1:k){
  Z[[cp]] <-colMeans(var[which(vector_cluster==cp),],na.rm=T)/10
}

datitosERA5 <- data.frame(Longitude=rep(longitude,k),
                          Latitude=rep(latitude,k),
                          Z = unlist(Z),
                          CP = rep(1:k,each=length(unlist(Z))/k))
datitosERA5$CP <- factor(datitosERA5$CP,levels=c(1:k))

# p <- ggplot(datitosERA5,aes(x=Longitude,y=Latitude))  + geom_contour_fill(aes(z=Z),size=100)  + borders(colour = "grey22",size=0.3) + facet_wrap(~CP,nrow=1)+
#   scale_fill_gradientn(name=expression("PSL (hPa)"),limits=c(10000,10500),
#                        colours=rev(brewer.pal(11,"Spectral")),aesthetics=c("fill"),
#                        na.value = "white") + ggtitle("ERA5") +
#   coord_quickmap(xlim=c(min(datitosERA5$Longitude),max(datitosERA5$Longitude)),ylim=c(min(datitosERA5$Latitude),max(datitosERA5$Latitude)),expand = F) +
#   xlab("Longitude") + ylab("Latitude") + theme_bw() +#+ ggtitle("Patterns") +
#   geom_contour2(aes(z=Z),  color="black", na.rm = T, binwidth=50)+
#   geom_text_contour(aes(z=Z),stroke = 0.10, rotate = T, check_overlap = T, binwidth=50,size=2.5)
# ggsave(p,filename=paste(OUTPUTS,"SpatialPatterns_Ward_SLP_",dom,"_","ERA5","_1x1_",k,"reordered_CPs.png",sep=""),device="png",
#        dpi=600,width=2*k,height = 3)

lonERA <- longitude 
latERA <- latitude

################################################################################
#Start working with models
files <- list.files(pattern="2014.nc",path = "/esarchive/scratch/molmo/output/recipe_20231114_085303/preproc/diagnostic_2/psl_d1/")
files <- list.files(pattern="2014.nc",path = "/esarchive/scratch/molmo/output/recipe_20231114_085303/preproc/diagnostic_1/psl_d1/")

files2 <- files

files2[9] <- files[10]
files2[10] <- files[11]
files2[11] <- files[13]
files2[12] <- files[12]
files2[13] <- files[9]

files2[21] <- files[22]
files2[22] <- files[21]

files <- files2
filesW <- list.files(pattern="WardClustering",path = "/esarchive/scratch/molmo/output/")

datitos.gcm <- list() ; gcms <- NA

for(f in 1:length(files)){
  load(file = paste("/esarchive/scratch/molmo/output/",filesW[f],sep=""))
  
  nc <- nc_open(paste("/esarchive/scratch/molmo/output/recipe_20231114_085303/preproc/diagnostic_2/psl_d1/",files[f],sep=""))
  nc_lon <- ncvar_get(nc,"lon")
  nc_lat <- ncvar_get(nc,"lat")
  nc_var <- ncvar_get(nc,"psl")
  
  lon <- nc_lon
  for(i in 1:length(lon)){
    if(lon[i]>180){
      lon[i] <- lon[i]-360
    }
  }
  lat <- nc_lat  
  
 # nc_var <- nc_var[which(lon%in%unique(lonERA)),which(lat%in%unique(latERA)),]
  var <- t(matrix(nc_var,dim(nc_var)[1]*dim(nc_var)[2],dim(nc_var)[3]))
  #var <- scale(t(var),scale=T,center=T)
  
  message(gcm)
  
  vector_cluster <- cutree(ward_clustering,k)
  
  Zm <- list()
  for(cp in 1:k){
    Zm[[cp]] <- colMeans(var[which(vector_cluster==cp),],na.rm=T)/10
  }
  
    
  
  xmn <- min(lon[which(lon%in%unique(lonERA))]) ; xmx <- max(lon[which(lon%in%unique(lonERA))]) 
  ymn <- min(lat[which(lat%in%unique(latERA))]) ; ymx <- max(lat[which(lat%in%unique(latERA))])
  longitude=rep(lon,length(lat[which(lat%in%unique(latERA))]))
  latitude=rep(lat,each=length(lon[which(lon%in%unique(lonERA))]))
  
  datitos <- data.frame(Longitude=rep(longitude,k),
                        Latitude=rep(latitude,k),
                        Z = unlist(Zm),
                        CP = rep(1:k,each=length(unlist(Zm))/k))
  datitos$CP <- factor(datitos$CP,levels=c(1:k))
  
  #Ordeno las longitudes del GCM como en ERA5 para luego correlacionarlos / dist euclidea
  #Ahora viene el ordenamiento
  lonERA <- datitosERA5$Longitude[1:(dim(datitosERA5)[1]/k)]
  latERA <- datitosERA5$Latitude[1:(dim(datitosERA5)[1]/k)]
  
  patronGCM <- list()
  for(j in 1:k){
    patronGCM[[j]] <- NA
    for(i in 1:length(longitude)){
      patronGCM[[j]][i] <- datitos$Z[which(datitos$Longitude==lonERA[i] & datitos$Latitude==latERA[i] & datitos$CP==j)]
    }
  }
  
  for(j in 1:k){
    datitos$Z[which(datitos$CP==j)] <- patronGCM[[j]]
  }
  
  #Load orden
  load(paste(OUTPUTS,"orden_CPs_",dom,"_",gcm,"_",k,".Rdata",sep=""))
  
  #Reordenamiento
  datos <- vector_cluster
  for(i in 1:k){
    datos[which(vector_cluster==i)] <- orden[i]
  }
  
  #Temporal cycle
  calendario <- seq(as.Date("1950-01-01"),as.Date("2014-12-31"),by="day")
  fechas <- calendario[format(calendario,"%m-%d") != "02-29"]
  n_years <- length(fechas)/365
  
  if(length(datos)!=length(fechas)){
    datos <- datos[which(calendario%in%fechas)]
    rm(calendario,fechas)
  }
  
  datos2 = data.frame(group=as.numeric( datos),
                      day=rep(1:365,n_years))

  datitos2 <- datitos
  for(cp in 1:length(orden)){
    datitos2$CP[which(datitos$CP==cp)] <- orden[cp]  
  }
  datitos <- datitos2

  # p <- ggplot(datitos,aes(x=Longitude,y=Latitude))  + geom_contour_fill(aes(z=Z),size=100)  + borders(colour = "grey22",size=0.3) + facet_wrap(~CP,nrow=1)+
  #   scale_fill_gradientn(name=expression("PSL (hPa)"),limits=c(10000,10500),
  #                        colours=rev(brewer.pal(11,"Spectral")),aesthetics=c("fill"),
  #                        na.value = "white") + ggtitle(gcm) +
  #   coord_quickmap(xlim=c(min(datitosERA5$Longitude),max(datitosERA5$Longitude)),ylim=c(min(datitosERA5$Latitude),max(datitosERA5$Latitude)),expand = F) +
  #   xlab("Longitude") + ylab("Latitude") + theme_bw() +#+ ggtitle("Patterns") +
  #   geom_contour2(aes(z=Z),  color="black", na.rm = T, binwidth=50)+
  #   geom_text_contour(aes(z=Z),stroke = 0.10, rotate = T, check_overlap = T, binwidth=50,size=2.5)
  # ggsave(p,filename=paste(OUTPUTS,"SpatialPatterns_Ward_SLP_",dom,"_",gcm,"_1x1_",k,"reordered_CPs.png",sep=""),device="png",
  #        dpi=600,width=2*k,height = 3)

  datitos.gcm[[f]] <- datitos
  gcms[f] <- gcm
}
  
#Taylor diagrams
  png(filename=paste(OUTPUTS,"TaylorDiagram_SLP_CPs_",dom,"_",k,"_D3.png",sep=""),width = 4000,height=4000/5,res=300)
  par(mfrow=c(1,5))
  for(cp in 1:length(orden)){
    ref = datitosERA5$Z[which(datitosERA5$CP==cp)]
    
    for(f in 1:length(files)){
      mod = datitos.gcm[[f]]$Z[which(datitos.gcm[[f]]$CP==cp)]
        if(f==1){
          td(ref=ref,
             model=mod,
             col=rainbow(n=length(files))[f],main=paste("CP",cp,sep=""),
             text=NULL,pos.cor = T,coltext = rainbow(n=length(files))[f],
             pcex=1,sd.arcs = T,ref.sd = T,normalize=T,grad.corr.lines=c(0.1,0.2,0.3,0.4,0.5,0.6,0.7,0.8,0.9),gamma.col = "pink",ngamma = 7)
        }else{
          td(ref=ref,
             model=mod,
             col=rainbow(n=length(files))[f],main=paste("CP",cp,sep=""),add = T,
             text=NULL,pos.cor = T,coltext = rainbow(n=length(files))[f],
             pcex=1,sd.arcs = T,ref.sd = T,normalize=T,grad.corr.lines=c(0.1,0.2,0.3,0.4,0.5,0.6,0.7,0.8,0.9),gamma.col = "pink",ngamma = 7)
        }    }
  }
dev.off()

png(filename=paste(OUTPUTS,"TaylorDiagram_Legend_SLP_CPs_",dom,"_",k,".png",sep=""),width = 2500,height=2500,res=300)
x<-1:10; y1=x*x; y2=2*y1
plot(x, y1, type="b", pch=19, col="white", xlab="x", ylab="y")
legend(x=1,y=95,legend=gcms,
       col=rainbow(n=length(files)), lty=1, cex=0.8,border = T)
dev.off()
