library(ncdf4)
library(ggplot2)
library(gridExtra)
library(fields)
library(mapdata)
library(matrixStats)
library(metR)
library(RColorBrewer)

#Asignación espacial comparando el dominio más grande sobre Europa (D3)
rm(list=ls()) ; gc()

#Directory information
setwd("../../../scratch/")
OUTPUTS <- "/esarchive/scratch/molmo/CPs_SLP_ERA5/"

#Useful functions
png.save <- function(x,filename,main,xlab,ylab){
  png(filename = paste(filename,".png",sep=""))
  plot(x,type="o",main=main,xlab = xlab,ylab=ylab)
  dev.off()
}

#########################################################
#Load ERA5 data
message("Loading and preprocessing data...")
nc <- nc_open("psl_5022_1x1_D3.nc")
nc_lon <- ncvar_get(nc,"lon")
nc_lat <- ncvar_get(nc,"lat")
nc_var <- ncvar_get(nc,"psl") #x,y,time

#Selection of spatial domains
gridsize_x <- abs(nc_lon[2]-nc_lon[1])
gridsize_y <- abs(nc_lat[2]-nc_lat[1])

# domain1 <- list(lon <- which(nc_lon %in% seq(-15,15,by=gridsize_x)),
#                 lat <- which(nc_lat %in% seq(32,48,by=gridsize_y))) #red domain D1 #Spain
# 
# domain2 <- list(lon <- which(nc_lon %in% seq(-10,25,by=gridsize_x)),
#                 lat <- which(nc_lat %in% seq(33,45,by=gridsize_y))) #red domain D2 PEP Mediterranean
# 
# domain3 <- list(lon <- which(nc_lon %in% seq(-35,35,by=gridsize_x)),
#                 lat <- which(nc_lat %in% seq(20,70,by=gridsize_y))) #red domain D3 #Europe
# 
# domain <- domain2 

#Organize and Standardize data
var <- matrix(nc_var,dim(nc_var)[1]*dim(nc_var)[2],dim(nc_var)[3]) #space,time
var <- scale(t(var),center=T,scale=T)
rm(nc_var) ; gc()

#Load clustering information
do <- "D2"
dom <- do[1]
k=5 

#Preparing data
longitude=rep(nc_lon,length(nc_lat))
latitude=rep(nc_lat,each=length(nc_lon))

Z <- list() 

load(file = paste("WardCluster_Dendrogram_","D2","_50km.Rdata",sep=""))
vector_cluster <- cutree(ward_clustering,k)

for(cp in 1:k){
  Z[[cp]] <-colMeans(var[which(vector_cluster==cp),],na.rm=T)
}

datitosERA5 <- data.frame(Longitude=rep(longitude,k),
                          Latitude=rep(latitude,k),
                          Z = unlist(Z),
                          CP = rep(1:k,each=length(unlist(Z))/k))
datitosERA5$CP <- factor(datitosERA5$CP,levels=c(1:k))

################################################################################
#Start working with models
files <- list.files(pattern="2014.nc",path = "/esarchive/scratch/molmo/output/recipe_20231114_085303/preproc/diagnostic_2/psl_d1/")
files2 <- files
files2[9] <- files[10]
files2[10] <- files[11]
files2[11] <- files[13]
files2[12] <- files[12]
files2[13] <- files[9]

files2[21] <- files[22]
files2[22] <- files[21]

files <- files2
filesW <- list.files(pattern="WardClustering",path = "/esarchive/scratch/molmo/output/")

for(f in 1:length(files)){
  load(file = paste("/esarchive/scratch/molmo/output/",filesW[f],sep=""))
  
  nc <- nc_open(paste("/esarchive/scratch/molmo/output/recipe_20231114_085303/preproc/diagnostic_2/psl_d1/",files[f],sep=""))
  nc_lon <- ncvar_get(nc,"lon")
  nc_lat <- ncvar_get(nc,"lat")
  nc_var <- ncvar_get(nc,"psl")
  var <- matrix(nc_var,dim(nc_var)[1]*dim(nc_var)[2],dim(nc_var)[3])
  
  ###
  # #Calculo de anomalías
  # calendario <- seq(as.Date("1950-01-01"),as.Date("2014-12-31"),by="day")
  # fechas <- calendario[format(calendario,"%m-%d") != "02-29"]
  # 
  # if(dim(var)[2]!=length(fechas)){ #13149
  #   var <- varCP[,which(calendario%in%fechas)]
  #   }
  # 
  # for(x in 1:dim(var)[1]){
  #   ciclo <- matrix(var[x,],365,dim(var)[2]/365)
  #   ciclo <- rowMeans(ciclo,na.rm=T)
  # 
  #   var[x,] <- var[x,] - rep(ciclo,dim(var)[2]/365)
  # }
  # 
  ###
  var <- scale(t(var),scale=T,center=T)
  
  message("Reordering ",gcm)
  
  vector_cluster <- cutree(ward_clustering,k)
  #vector_cluster <- vector_cluster[which(calendario%in%fechas)]
  
  Zm <- list()
  for(cp in 1:k){
    Zm[[cp]] <- colMeans(var[which(vector_cluster==cp),],na.rm=T)
  }
  
  lon <- nc_lon
  for(i in 1:length(lon)){
    if(lon[i]>180){
      lon[i] <- lon[i]-360
    }
  }
  lat <- nc_lat    
  
  xmn <- min(lon) ; xmx <- max(lon) ; ymn <- min(lat) ; ymx <- max(lat)
  longitude=rep(lon,length(nc_lat))
  latitude=rep(seq(ymn,ymx,gridsize_x),each=length(nc_lon))
  
  datitos <- data.frame(Longitude=rep(longitude,k),
                        Latitude=rep(latitude,k),
                        Z = unlist(Zm),
                        CP = rep(1:k,each=length(unlist(Zm))/k))
  datitos$CP <- factor(datitos$CP,levels=c(1:k))
  
  # p <- ggplot(datitos,aes(x=Longitude,y=Latitude))  + geom_contour_fill(aes(z=Z),size=1)  + borders(colour = "grey22",size=0.3) + facet_wrap(~CP,nrow=1)+ 
  #   scale_fill_gradientn(name=expression("PSL (hPa)"),
  #                        colours=rev(brewer.pal(11,"Spectral")),aesthetics=c("fill"),
  #                        na.value = "white") + ggtitle(gcm) +
  #   coord_quickmap(xlim=c(min(datitosERA5$Longitude),max(datitosERA5$Longitude)),ylim=c(min(datitosERA5$Latitude),max(datitosERA5$Latitude)),expand = F) +
  #   xlab("Longitude") + ylab("Latitude") + theme_bw() +#+ ggtitle("Patterns") + 
  #   geom_contour2(aes(z=Z),  color="black", na.rm = T, binwidth=5)+
  #   geom_text_contour(aes(z=Z),stroke = 0.10, rotate = T, check_overlap = T, binwidth=5,size=2.5)
  # ggsave(p,filename=paste(OUTPUTS,"SpatialPatterns_Ward_SLP_",dom,"_",gcm,"_1x1_",k,"reordered_CPs.png",sep=""),device="png",
  #        dpi=600,width=2*k,height = 3)
  
  #Ordeno las longitudes del GCM como en ERA5 para luego correlacionarlos / dist euclidea
  #Ahora viene el ordenamiento
  lonERA <- datitosERA5$Longitude[1:3500]
  latERA <- datitosERA5$Latitude[1:3500]
  
  patronGCM <- list()
  for(j in 1:k){
    patronGCM[[j]] <- NA
    for(i in 1:length(longitude)){
      patronGCM[[j]][i] <- datitos$Z[which(datitos$Longitude==lonERA[i] & datitos$Latitude==latERA[i] & datitos$CP==j)]
    }
  }
  
  #Matriz de distancias
  distance <- matrix(NA,k,k)
  for(i in 1:k){
    for(j in 1:k){
      distance[i,j] <- dist(rbind(Z[[i]],patronGCM[[j]]),method = "euclidean")
    }
  }
  
  orden <- rep(0,k)
  for(i in 1:5){
    closest_index <- which.min(distance[,i])
    
    if (closest_index %in% orden){
      left <- c(1:5)[-which(c(1:5)%in%orden)]
      closest_index <- left[which.min(distance[left,i])]
    }
    orden[i] <- closest_index  
  }
  
  if(gcm %in% c("ACCESS-CM2","CMCC-CM2-HR4","EC-Earth3-AerChem","KIOST-ESM","IPSL-CM6A-LR","MPI-ESM1-2-HR","MPI-ESM1-2-LR","NorESM2-LM","NorESM2-MM","TaiESM1")){ 
    orden2 <- orden
    orden2[which(orden==1)] <- 3
    orden2[which(orden==3)] <- 1
    orden <- orden2
  }
  save(orden,file=paste(paste(OUTPUTS,"orden_CPs_",dom,"_",gcm,"_",k,".Rdata",sep="")))

  #Reordenamiento
  datos <- vector_cluster
  for(i in 1:k){
    datos[which(vector_cluster==i)] <- orden[i]
  }
  
  #Temporal cycle
  calendario <- seq(as.Date("1950-01-01"),as.Date("2014-12-31"),by="day")
  fechas <- calendario[format(calendario,"%m-%d") != "02-29"]
  n_years <- length(fechas)/365
  
  if(length(datos)!=length(fechas)){
    datos <- datos[which(calendario%in%fechas)]
    rm(calendario,fechas)
  }
  
  datos2 = data.frame(group=as.numeric( datos),
                      day=rep(1:365,n_years))
  
  # if(gcm %in% c("ACCESS-CM2","CMCC-CM2-HR4","EC-Earth3-AerChem","KIOST-ESM","IPSL-CM6A-LR","MPI-ESM1-2-HR","MPI-ESM1-2-LR","NorESM2-LM","NorESM2-MM","TaiESM1")){ 
  #   datos3 <- datos2
  #   datos3$group[which(datos2$group==1)] <- 3
  #   datos3$group[which(datos2$group==3)] <- 1
  #   datos2 <- datos3
  # }
  # 
  # if(gcm %in% c()){ 
  #   datos3 <- datos2
  #   datos3$group[which(datos2$group==3)] <- 5
  #   datos3$group[which(datos2$group==5)] <- 3
  #   datos2 <- datos3
  # }
  
  value <- matrix(NA,365,k)
  for(i in 1:365){
    for(j in 1:k){
      value[i,j] <- length(which(datos2$group==j & datos2$day==i))/n_years*100
    }
  }
  
  palette <- brewer.pal(n=7,name = "Set1")
  
  datos = data.frame(day=rep(1:365,k),frequency=as.numeric(value),group=factor(rep(1:k,each=365),levels=c(1:k)))
  p <- ggplot(data=datos,aes(x=day,y=frequency,fill=group)) + geom_col() + coord_cartesian(expand=F)+
    scale_fill_manual(values=palette[1:k]) + theme_bw() + ggtitle(gcm) + 
    scale_x_continuous(labels = c("J","F","M","A","M","J","J","A","S","O","N","D"),breaks = seq(15,365,by=30))
  ggsave(p,filename=paste(OUTPUTS,"relativefrequency_CPs_",dom,"_",gcm,"_",k,"_reorderedD3_CPs_ultimo.png",sep=""),device="png",
         dpi=600,width=4.84,height = 3.95)
  gc()
  
}
